@@ -1,3 +1,6 @@
+import sys
+import io
+
 from copy import deepcopy
 from collections import namedtuple
 from contextlib import contextmanager
@@ -18,18 +21,15 @@ except ImportError:
     mysql = None
 
 
-__version__ = "0.4.0"
+__version__ = "0.5.1"
 __all__ = [
-    "ClauseError",
     "Database",
-    "DatabaseError",
-    "EstoultError",
     "Field",
-    "FieldError",
     "fn",
     "op",
+    "Schema",
+    "QF",
     "Query",
-    "QueryError",
 ]
 
 
@@ -53,97 +53,138 @@ class DatabaseError(EstoultError):
     pass
 
 
-Clause = namedtuple("Clause", ["clause", "params"])
+_sql_ops = {
+    "eq": "=",
+    "lt": "<",
+    "le": "<=",
+    "gt": ">",
+    "ge": ">=",
+    "ne": "<>",
+}
+
+
+def _parse_arg(arg):
+    if isinstance(arg, Clause):
+        return arg
+    elif isinstance(arg, Field):
+        return str(arg), ()
+    elif isinstance(arg, Query):
+        return arg._query, arg._params
+    elif isinstance(arg, list) or isinstance(arg, tuple):
+        placeholders = ", ".join(["%s"] * len(arg))
+        return placeholders, tuple(arg)
+
+    return "%s", (arg,)
+
+
+def _parse_args(func):
+    def wrapper(*args):
+        return func(*[_parse_arg(a) for a in args])
+
+    return wrapper
 
 
 def _strip(string):
-    return string.rstrip(" ").rstrip(",").rstrip("and")
+    string = string.rstrip(" ,")
+
+    if string.endswith("and"):
+        string = string[:-3]
+
+    return string
+
+
+def _make_op(operator):
+    @_parse_args
+    def wrapper(lhs, rhs):
+        return Clause(f"({lhs[0]}) {operator} ({rhs[0]})", tuple(lhs[1] + rhs[1]))
+
+    return wrapper
+
+
+def _make_fn(name):
+    def wrapper(*args):
+        return Clause(f"{name}({str(', '.join([str(a) for a in args]))})", ())
+
+    return wrapper
+
+
+class ClauseMetaclass(type):
+    def __new__(cls, clsname, bases, attrs):
+        # Add op overloading
+        for name, operator in _sql_ops.items():
+            attrs[f"__{name}__"] = staticmethod(_make_op(operator))
+
+        return super(ClauseMetaclass, cls).__new__(cls, clsname, bases, attrs)
+
+
+class Clause(namedtuple("Clause", ["clause", "params"]), metaclass=ClauseMetaclass):
+    def __str__(self):
+        return self.clause
+
+    def __hash__(self):
+        return hash(str(self))
+
+    def __eq__(self, comp):
+        return str(self) == comp
 
 
 class OperatorMetaclass(type):
-
-    sql_ops = {
-        "eq": "=",
-        "lt": "<",
-        "le": "<=",
-        "gt": ">",
-        "ge": ">=",
-        "ne": "<>",
-    }
-
-    @staticmethod
-    def make_fn(operator):
-        def op_fn(field, value):
-            return Clause(f"{field} {operator} %s", (value,))
-
-        return op_fn
-
     def __new__(cls, clsname, bases, attrs):
-        for name, operator in cls.sql_ops.items():
-            attrs[name] = OperatorMetaclass.make_fn(operator)
+        for name, operator in _sql_ops.items():
+            attrs[name] = staticmethod(_make_op(operator))
 
-        return super(OperatorMetaclass, cls).__new__(cls, clsname, bases, attrs)
+        c = super(OperatorMetaclass, cls).__new__(cls, clsname, bases, attrs)
+
+        c.add_op("or_", "or")
+        c.add_op("and_", "and")
+        c.add_op("in_", "in")
+        c.add_op("like", "like")
+
+        return c
 
 
 class op(metaclass=OperatorMetaclass):
-    @staticmethod
-    def or_(cond_1, cond_2):
-        return Clause(
-            f"({_strip(cond_1[0])} or {_strip(cond_2[0])})", (*cond_1[1], *cond_2[1]),
-        )
+    @classmethod
+    def add_op(cls, name, op):
+        """
+        Adds an operator to the module.
+
+        :param name: What the name of the function should be called.
+        :type name: str
+        :param op: The SQL operator it is turned into.
+        :type op: str
+        """
+
+        def func(lhs, rhs):
+            fn = _make_op(op)
+            return fn(lhs, rhs)
+
+        setattr(cls, name, staticmethod(func))
 
     @staticmethod
-    def and_(cond_1, cond_2):
-        return Clause(
-            f"({_strip(cond_1[0])} and {_strip(cond_2[0])})", (*cond_1[1], *cond_2[1]),
-        )
+    @_parse_args
+    def ilike(lhs, rhs):
+        # Does a case insensitive `like`. Only postgres has this operator,
+        # but we can hack it together for the others
+        if psycopg2:
+            return Clause(f"({lhs[0]}) ilike ({rhs[0]})", (lhs[1] + rhs[1]))
+
+        return Clause(f"lower({lhs[0]}) like lower({rhs[0]})", (lhs[1] + rhs[1]))
 
     @staticmethod
-    def in_(field, value):
-        if isinstance(value, Query):
-            return Clause(f"{field} in ({str(value)})", ())
-
-        if isinstance(value, list) or isinstance(value, tuple):
-            placeholders = ", ".join(["%s"] * len(value))
-            return Clause(f"{field} in ({placeholders})", (value,))
-
-        raise ClauseError("`in` value can only be `subquery`, `list`, or `tuple`")
+    @_parse_args
+    def not_(arg):
+        return Clause(f"not ({arg[0]})", (arg[1]))
 
     @staticmethod
-    def like(field, value):
-        arg = f"%{value}%"
-        return Clause(f"{field} like %s", (arg,))
+    @_parse_args
+    def is_null(arg):
+        return Clause(f"({arg[0]}) is null", (arg[1]))
 
     @staticmethod
-    def ilike(field, value):
-        arg = f"%{value}%"
-        return Clause(f"{field} ilike %s", (arg,))
-
-    @staticmethod
-    def not_(field):
-        return Clause(f"not {field}", ())
-
-    @staticmethod
-    def is_null(field):
-        return Clause(f"{field} is null", ())
-
-    @staticmethod
-    def not_null(field):
-        return Clause(f"{field} is not null", ())
-
-
-class FunctionClauseMetaclass(type):
-    def __new__(cls, clsname, bases, attrs):
-        # Add op overloading
-        for name, operator in OperatorMetaclass.sql_ops.items():
-            attrs[f"__{name}__"] = OperatorMetaclass.make_fn(operator)
-
-        return super(FunctionClauseMetaclass, cls).__new__(cls, clsname, bases, attrs)
-
-
-class FunctionClause(Clause, metaclass=FunctionClauseMetaclass):
-    def __str__(self):
-        return self.clause
+    @_parse_args
+    def not_null(arg):
+        return Clause(f"({arg[0]}) is not null", (arg[1]))
 
 
 class FunctionMetaclass(type):
@@ -157,59 +198,90 @@ class FunctionMetaclass(type):
         "concat",
     ]
 
-    @staticmethod
-    def make_sql_fn(name):
-        def sql_fn(*args):
-            return FunctionClause(
-                f"{name}({str(', '.join([str(a) for a in args]))})", ()
-            )
-
-        return sql_fn
-
     def __new__(cls, clsname, bases, attrs):
         for f in cls.sql_fns:
-            attrs[f] = FunctionMetaclass.make_sql_fn(f)
-
-        # Add op overloading
-        for name, operator in OperatorMetaclass.sql_ops.items():
-            attrs[f"__{name}__"] = OperatorMetaclass.make_fn(operator)
+            attrs[f] = staticmethod(_make_fn(f))
 
         return super(FunctionMetaclass, cls).__new__(cls, clsname, bases, attrs)
 
 
 class fn(metaclass=FunctionMetaclass):
-    @staticmethod
-    def alias(field, value):
-        return FunctionClause(f"{field} as {value}", ())
+    @classmethod
+    def add_fn(cls, name, sql_fn):
+        """
+        Adds an additional function to the module.
+
+        :param name: What the name of the function should be called.
+        :type name: str
+        :param sql_fn: The SQL function it is turned into.
+        :type sql_fn: str
+        """
+
+        def func(*args):
+            fn = _make_fn(sql_fn)
+            return fn(*args)
+
+        setattr(cls, name, staticmethod(func))
 
     @staticmethod
-    def cast(field, value):
-        return FunctionClause(f"cast({field} as {value})", ())
+    def alias(lhs, rhs):
+        s, p = _parse_arg(lhs)
+        return Clause(f"{s} as {rhs}", tuple(p))
+
+    @staticmethod
+    def cast(lhs, rhs):
+        s, p = _parse_arg(lhs)
+        return Clause(f"cast({s} as {rhs})", tuple(p))
 
     @staticmethod
     def wild(schema):
-        return FunctionClause(f"{schema.__tablename__}.*", ())
+        """
+        Select a schema with wildcard.
+
+        :param schema: The schema.
+        :type schema: Schema
+        """
+        return Clause(f"{schema.__tablename__}.*", ())
 
 
 class FieldMetaclass(type):
     def __new__(cls, clsname, bases, attrs):
         # Add op overloading
-        for name, operator in OperatorMetaclass.sql_ops.items():
-            attrs[f"__{name}__"] = OperatorMetaclass.make_fn(operator)
+        for name, operator in _sql_ops.items():
+            attrs[f"__{name}__"] = _make_op(operator)
 
         return super(FieldMetaclass, cls).__new__(cls, clsname, bases, attrs)
 
 
 class Field(metaclass=FieldMetaclass):
-    def __init__(self, type, name, **kwargs):
+    """
+    A schema field, analogous to columns on a table.
+
+    :param type: Basic datatype of the field, used to cast values into the database.
+    :type type: type
+    :param name: The column name of the field, will default to the variable name of the
+                 field.
+    :type name: str, optional
+    :param caster: A specified caster type/function for more extensive casting.
+    :type caster: callable, optional
+    :param null: Field allows nulls.
+    :type null: bool, optional
+    :param default: Default value.
+    :type default: optional
+    :param primary_key: Field is the primary key.
+    :type primary_key: bool, optional
+    """
+
+    def __init__(
+        self, type, name=None, caster=None, null=True, default=None, primary_key=False
+    ):
         self.type = type
         self.name = name
 
-        self.caster = kwargs.get("caster")
-
-        self.null = kwargs.get("null", True)
-        self.default = kwargs.get("default")
-        self.primary_key = kwargs.get("primary_key") is True
+        self.caster = caster
+        self.null = null
+        self.default = default
+        self.primary_key = primary_key
 
     @property
     def full_name(self):
@@ -221,8 +293,32 @@ class Field(metaclass=FieldMetaclass):
     def __hash__(self):
         return hash(str(self))
 
-    def __eq__(self, comp):
-        return str(self) == comp
+    def __repr__(self):
+        return (
+            f"<Field {self.type} name={self.name} caster={self.caster} "
+            f"null={self.null} default={self.default} "
+            f"primary_key={self.primary_key}>"
+        )
+
+
+class QF(Field):
+    """
+    An extra user defined field used for queries.
+    This is mainly needed for referencing aliases.
+
+    :param name: The name of the field.
+    :type name: str
+    """
+
+    def __init__(self, name):
+        self.name = name
+
+    @property
+    def full_name(self):
+        return f"{self.name}"
+
+    def __repr__(self):
+        return f"<QF name={self.name}>"
 
 
 class SchemaMetaclass(type):
@@ -239,13 +335,17 @@ class SchemaMetaclass(type):
 
         c = super(SchemaMetaclass, cls).__new__(cls, clsname, bases, attrs)
 
-        # Add schema to fields
         for key in dir(c):
             f = getattr(c, key)
 
             if isinstance(f, Field):
+                # Reference schema in fields
                 f.schema = c
 
+                # Set name to var reference
+                if f.name is None:
+                    f.name = key
+
         return c
 
     @property
@@ -292,15 +392,22 @@ class Schema(metaclass=SchemaMetaclass):
         for field in cls.fields:
             value = None
 
-            if field.default is not None:
-                value = field.default
-
             try:
+                # Try to get the value from the row
+                # In a try/catch so we can tell the difference between
+                # >>> row["field"] == None  # Field is set to `None`
+                # >>> row.get("field") == None  # Field is not set
                 value = row[field.name]
             except KeyError:
-                if updating is True or field.name == cls.pk.name:
+                # We didn't pass this field in, so move on if we're updating
+                if updating is True:
                     continue
 
+            # Apply a default if we are inserting
+            if value is None and updating is False and field.default is not None:
+                value = field.default
+
+            # Cast the value
             if value is not None:
                 value = (
                     field.type(value) if field.caster is None else field.caster(value)
@@ -328,9 +435,7 @@ class Schema(metaclass=SchemaMetaclass):
         return changeset
 
     @classmethod
-    def casval(cls, row):
-        updating = row.get(cls.pk.name) is not None
-
+    def _casval(cls, row, updating):
         changeset = cls._cast(updating, row)
         changeset = cls._validate(updating, changeset)
 
@@ -342,19 +447,19 @@ class Schema(metaclass=SchemaMetaclass):
 
     @classmethod
     def insert(cls, obj):
-        changeset = cls.casval(obj)
+        changeset = cls._casval(obj, updating=False)
 
         params = list(changeset.values())
         fields = ", ".join(changeset.keys())
         placeholders = ", ".join(["%s"] * len(changeset))
 
-        sql = f"insert into {cls.__tablename__} (%s) values (%s)\n" % (
+        sql = f"insert into {cls.__tablename__} (%s) values (%s)" % (
             fields,
             placeholders,
         )
 
         if psycopg2 is not None:
-            sql += f"returning {cls.pk.name}\n"
+            sql += f" returning {cls.pk.name}"
 
         return cls._database_.insert(_strip(sql), params)
 
@@ -362,7 +467,7 @@ class Schema(metaclass=SchemaMetaclass):
     def update(cls, old, new):
         # This updates a single row only, if you want to update several
         # use `update` in `Query`
-        changeset = cls.casval({**old, **new})
+        changeset = cls._casval({**old, **new}, updating=True)
         sql = f"update {cls.__tablename__} set "
         params = []
 
@@ -415,7 +520,7 @@ class QueryMetaclass(type):
     def make_join_fn(join_type):
         def join_fn(self, schema, on):
             q = f"{str(on[0])} = {str(on[1])}"
-            self._query += f"{join_type} {schema.__tablename__} on {q}\n"
+            self._add_node(f"{join_type} {schema.__tablename__} on {q}", ())
             return self
 
         return join_fn
@@ -427,43 +532,69 @@ class QueryMetaclass(type):
         return super(QueryMetaclass, cls).__new__(cls, clsname, bases, attrs)
 
 
+Node = namedtuple("Node", ["node", "params"])
+
+
 class Query(metaclass=QueryMetaclass):
     def __init__(self, schema):
         self.schema = schema
 
         self._method = None
-        self._query = ""
-        self._params = []
+        self._nodes = []
+
+    def _add_node(self, node, params):
+        self._nodes.append(Node(_strip(node), params))
+
+    @property
+    def _query(self):
+        return " ".join([x.node for x in self._nodes])
+
+    @property
+    def _params(self):
+        return tuple([p for x in self._nodes for p in x.params])
 
     def select(self, *args):
         self._method = "select"
 
-        if len(args) < 1:
-            args = "*"
-        else:
-            args = ", ".join([str(a) for a in args])
+        query = ""
+        params = []
 
-        self._query = f"select {args} from {self.schema.__tablename__}\n"
+        if len(args) < 1:
+            query += "*"
+        else:
+            for arg in args:
+                if isinstance(arg, Clause):
+                    string, p = arg
+                    query += f"{string}, "
+                    params.extend(p)
+                else:
+                    query += f"{arg}, "
+
+        self._add_node(
+            f"select {_strip(query)} from {self.schema.__tablename__}", params
+        )
 
         return self
 
     def update(self, changeset):
         self._method = "sql"
-        self._query = f"update {self.schema.__tablename__} set "
 
-        changeset = self.schema.casval(changeset)
+        changeset = self.schema._casval(changeset, updating=True)
+
+        query = ""
+        params = []
 
         for key, value in changeset.items():
-            self._query += f"{key} = %s, "
-            self._params.append(value)
+            query += f"{key} = %s, "
+            params.append(value)
 
-        self._query = f"{_strip(self._query)}\n"
+        self._add_node(f"update {self.schema.__tablename__} set {query}", params)
 
         return self
 
     def delete(self):
         self._method = "sql"
-        self._query = f"delete from {self.schema.__tablename__}\n"
+        self._add_node(f"delete from {self.schema.__tablename__}", ())
         return self
 
     def get(self, *args):
@@ -477,54 +608,66 @@ class Query(metaclass=QueryMetaclass):
         return self
 
     def union(self):
-        self._query += "union\n"
+        self._add_node("union", ())
         return self
 
     def where(self, *clauses):
-        self._query += "where "
+        query = ""
+        params = []
 
         for clause in clauses:
-            string, params = clause
+            string, p = clause
 
             # We can always add an `and` to the end cus it get stripped off ;)
-            self._query += f"{string} and "
-            self._params.extend(params)
+            query += f"{string} and "
+            params.extend(p)
 
-        self._query = f"{_strip(self._query)}\n"
+        self._add_node(f"where {query}", params)
 
         return self
 
     def limit(self, *args):
         # Example: .limit(1) or limit(1, 2)
         if len(args) == 1:
-            self._query += "limit %s\n"
+            self._add_node("limit %s", args)
         elif len(args) == 2:
             # `offset` works in mysql and postgres
-            self._query += "limit %s offset %s\n"
+            self._add_node("limit %s offset %s", args)
         else:
             raise QueryError("`limit` has too many arguments")
 
-        self._params.extend(args)
-
         return self
 
     def order_by(self, *args):
         # Example: .order_by(Frog.id, {Frog.name: "desc"})
+        query = "order by "
         params = []
 
         for a in args:
+            v = None
+
             if isinstance(a, dict):
-                for k, v in a.items():
-                    if v != "asc" and v != "desc":
-                        raise QueryError("Value must be 'asc' or 'desc'")
+                k, v = next(iter(a.items()))
 
-                    params.extend([str(k), v])
+                if v != "asc" and v != "desc":
+                    raise QueryError("Value must be 'asc' or 'desc'")
             else:
-                params.extend([str(a, "asc")])
+                k = a
 
-        s = ", ".join(["%s %s" for a in args]) % tuple(params)
+            if isinstance(k, Clause):
+                c, p = _parse_arg(k)
+                query += "%s " % c
+                params.extend(p)
+            elif isinstance(k, Field):
+                query += f"{k} "
+            else:
+                query += "%s "
+                params.append(str(k))
 
-        self._query += f"order by {s}\n"
+            if v:
+                query += f"{v}, "
+
+        self._add_node(f"{query}", params)
 
         return self
 
@@ -536,13 +679,14 @@ class Query(metaclass=QueryMetaclass):
         return deepcopy(self)
 
     def __str__(self):
-        return f"""
-            {(self.schema._database_
-                .mogrify(self._query, self._params)
-                .decode("utf-8"))}
-        """.replace(
-            "\n", " "
-        ).strip()
+        return (
+            self.schema._database_.mogrify(self._query, self._params)
+            .decode("utf-8")
+            .strip()
+        )
+
+    def __repr__(self):
+        return f'<Query query="{self._query}" params={self._params}>'
 
 
 def _replace_placeholders(func):
@@ -588,6 +732,7 @@ class Database:
     def connect(self):
         self._conn = self._connect()
 
+    @property
     def conn(self):
         return self._conn
 
@@ -636,12 +781,6 @@ class Database:
     def sql(self, query, params):
         return self._execute(query, params)
 
-    @_get_connection
-    def mogrify(self, query, params):
-        with self.atomic(commit=False):
-            self._execute(query, params)
-            return self.cursor._executed
-
     @_get_connection
     def select(self, query, params):
         self._execute(query, params)
@@ -677,6 +816,12 @@ class MySQLDatabase(Database):
     def _connect(self):
         return mysql.connect(*self.cargs, **self.ckwargs)
 
+    @_get_connection
+    def mogrify(self, query, params):
+        with self.atomic(commit=False):
+            self._execute(query, params)
+            return self.cursor._executed
+
 
 class PostgreSQLDatabase(Database):
     def __init__(self, *args, **kwargs):
@@ -700,3 +845,23 @@ class SQLiteDatabase(Database):
 
     def _connect(self):
         return sqlite3.connect(*self.cargs, **self.ckwargs)
+
+    @_get_connection
+    def mogrify(self, query, params):
+        with self.atomic(commit=False):
+            # SQLite doesn't have a thing to return the executed statement.
+            # But we **can** print it! So just capture that! :) FML
+
+            # redirect sys.stdout to a buffer
+            self.conn.set_trace_callback(print)
+            stdout = sys.stdout
+            sys.stdout = io.StringIO()
+
+            self._execute(query, params)
+
+            # get output and restore sys.stdout
+            output = sys.stdout.getvalue()
+            sys.stdout = stdout
+            self.conn.set_trace_callback(None)
+
+            return output.encode("utf-8")
